CREATE PROCEDURE [dbo].[AllGroupAnswersByDemographic]

@SurveyNumber int,
@Demographic int

AS

SET NOCOUNT ON

DECLARE @Temp table
(
GroupNumber int,
Answer int,
PNum int
)

INSERT @Temp
	SELECT Q.GroupNumber, A.Answer, A.PNum  
	FROM SurveyQuestions Q INNER JOIN dbo.Answers A ON Q.QuestionID = A.QuestionID 
	WHERE Q.Qtype = 1 AND Q.SurveyNumber = @SurveyNumber AND A.PNum IN (SELECT PNum FROM dbo.Participants WHERE SurveyNumber = @SurveyNumber) 



DECLARE @Demo table
(
PNUM int,
DemAnswer int, 
ResponseText varchar(200),
tid int identity
)

INSERT @Demo	
	SELECT A.PNum, A.Answer, '' FROM Answers A WHERE A.QuestionID = @Demographic

DECLARE @Possibles table
(
Sequence int,
ResponseText varchar(200),
tid int identity 
) 

INSERT @Possibles
	SELECT Sequence, ResponseText FROM dbo.surveyResponses WHERE QuestionID = @Demographic 

DECLARE @Temp2 table
(
GroupNumber int,
GroupText varchar(200) ,
AverageAnswer Decimal(6,2),
ResponseText varchar(200),
Sequence int
)

DECLARE @Start int, @Stop int, @Answer int, @DemoFldText varchar(200), @GroupText varchar(200) ,@DemAnswer int, @ResponseText varchar(200), @NumInDemo int
SET @Start = 1
SELECT @Stop = COUNT(*) + 1 FROM dbo.SurveyResponses WHERE QuestionID = @Demographic
WHILE @Start < @Stop
	BEGIN
		SELECT @DemoFldText = ResponseText FROM dbo.SurveyResponses WHERE QuestionID = @Demographic AND Sequence = @Start  
	 
		INSERT @Temp2
			SELECT T.GroupNumber, G.GroupText, Cast(AVG(CAST(Answer as decimal(10,4))) as Decimal(10,2)), @DemoFldText, @Start FROM @Temp T INNER JOIN QuestionGroup G ON T.GroupNumber = G.GroupNumber WHERE pNum IN(SELECT PNum FROM dbo.Answers WHERE CAST(Answer as int) = @Start AND QuestionID = @Demographic) GROUP BY T.GroupNumber, G.GroupText

		SET @Start = @Start + 1
	END

	
DECLARE @ResponseTable Table
(
	ResponseText varchar(200),
	Sequence int,
	tid int identity
)
IF EXISTS(SELECT ResponseText FROM dbo.SurveyResponses WHERE QuestionID = @Demographic and NOT ResponseText IN(SELECT ResponseText FROM @Temp2))
	BEGIN

		INSERT @ResponseTable
			SELECT ResponseText, Sequence FROM dbo.SurveyResponses WHERE QuestionID = @Demographic and NOT ResponseText IN(SELECT ResponseText FROM @Temp2) 
		SET @Start = 1
		SELECT @Stop = COUNT (*) + 1 FROM @ResponseTable
		
		WHILE @Start < @Stop	
			BEGIN
				SELECT @DemoFldText = ResponseText, @Answer = Sequence FROM @ResponseTable WHERE tid = @Start
				INSERT @Temp2
					SELECT DISTINCT G.GroupNumber,GroupText, NULL, @DemoFldText, @Answer FROM dbo.QuestionGroup G INNER JOIN dbo.SurveyQuestions Q ON G.GroupNumber = Q.GroupNumber WHERE Q.QType = 1
				
				SET @Start = @Start + 1
			END 

	END 


	SET @Start = 1
SELECT @Stop = COUNT(*) + 1 FROM @Possibles

WHILE @Start < @Stop	
	BEGIN
		SELECT @DemAnswer = Sequence, @ResponseText = ResponseText FROM @Possibles WHERE tid = @Start
		SELECT @NumInDemo = COUNT(*) FROM dbo.Participants P INNER JOIN @Demo D ON P.PNum = D.PNUM WHERE SurveyNumber = 1 and D.DemAnswer = @DemAnswer
		IF @NumInDemo > 0
			BEGIN
				INSERT @Temp2
					SELECT 50, 'Over All Average', Cast(AVG(CAST(Answer as decimal(10,4))) as Decimal(10,2)), @ResponseText , @DemAnswer FROM dbo.Answers A INNER JOIN dbo.SurveyQuestions Q ON A.QuestionID = Q.QuestionID 
					WHERE Q.QType = 1 AND A.PNum IN(SELECT P.PNum FROM dbo.Participants P INNER JOIN @Demo D ON P.PNum = D.PNUM WHERE SurveyNumber = 1 and D.DemAnswer = @DemAnswer)
				
			END
		ELSE
			BEGIN
				INSERT @Temp2 SELECT 50,'Over All Average', 0, @ResponseText, @DemAnswer
			END
		SET @Start = @Start + 1
	END

SELECT GroupNumber, GroupText, AverageAnswer, ResponseText FROM @Temp2  ORDER BY GroupNumber, Sequence

GO




CREATE PROCEDURE [dbo].[PivotedAllGroupsByDemographic]

@SurveyNumber int,
@Demographic int

AS

SET NOCOUNT ON

CREATE TABLE #Data
(
	GroupNumber int,
	GroupText varchar(200), 
	AverageAnswer float,
	ResponseText varchar(200)
)
INSERT #Data
	EXECUTE AllGroupAnswersByDemographic @SurveyNumber, @Demographic

DECLARE @PivotQuery NVARCHAR(MAX)
DECLARE @Cols varchar(max)
SET @Cols =  (SELECT CHAR(13) + CHAR(9) + ',MAX(CASE WHEN ResponseText = ' + QUOTENAME(ResponseText,'''') + ' THEN AverageAnswer ELSE 0 END) AS ' + QUOTENAME(ResponseText)
        FROM dbo.SurveyResponses 
        WHERE QuestionID = @Demographic 
        ORDER BY [Sequence]  FOR XML PATH(''), TYPE).value('.', 'NVARCHAR(MAX)')

--SELECT @Cols
	
SET @PivotQuery = 'SELECT GroupText '+ @Cols +' FROM #Data GROUP BY GroupText, GroupNumber ORDER BY GroupNumber'
--select @PivotQuery
EXECUTE(@PivotQuery)

DROP TABLE #Data

GO